@@ -15,6 +15,18 @@
 
 using namespace Baloo;
 
+namespace {
+    // TODO KF6 -- remove/combine with started/finished DBus signal
+    void sendChangedSignal(const QStringList& updatedFiles)
+    {
+        auto message = QDBusMessage::createSignal(QStringLiteral("/files"),
+                                                  QStringLiteral("org.kde"),
+                                                  QStringLiteral("changed"));
+        message.setArguments({updatedFiles});
+        QDBusConnection::sessionBus().send(message);
+    }
+}
+
 FileContentIndexer::FileContentIndexer(uint batchSize,
         FileContentIndexerProvider* provider,
         uint& finishedCount, QObject* parent)
@@ -41,11 +53,7 @@ void FileContentIndexer::run()
     ExtractorProcess process;
     connect(&process, &ExtractorProcess::startedIndexingFile, this, &FileContentIndexer::slotStartedIndexingFile);
     connect(&process, &ExtractorProcess::finishedIndexingFile, this, &FileContentIndexer::slotFinishedIndexingFile);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-    m_stop.store(false);
-#else
     m_stop.storeRelaxed(false);
-#endif
     auto batchSize = m_batchSize;
     while (true) {
         //
@@ -53,11 +61,7 @@ void FileContentIndexer::run()
         // cause then we will keep fetching the same N files again and again.
         //
         QVector<quint64> idList = m_provider->fetch(batchSize);
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-        if (idList.isEmpty() || m_stop.load()) {
-#else
         if (idList.isEmpty() || m_stop.loadRelaxed()) {
-#endif
             break;
         }
         QEventLoop loop;
@@ -76,11 +80,7 @@ void FileContentIndexer::run()
         loop.exec();
         batchSize = idList.size();
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-        if (hadErrors && !m_stop.load()) {
-#else
         if (hadErrors && !m_stop.loadRelaxed()) {
-#endif
             if (batchSize == 1) {
                 auto failedId = idList.first();
                 auto ok = m_provider->markFailed(failedId);
@@ -92,10 +92,16 @@ void FileContentIndexer::run()
             } else {
                 batchSize /= 2;
             }
-            // reset to old value - nothing comitted
+            m_updatedFiles.clear();
+            // reset to old value - nothing committed
             m_finishedCount = batchStartCount;
             process.start();
         } else {
+            // Notify some metadata may have changed
+            sendChangedSignal(m_updatedFiles);
+            m_updatedFiles.clear();
+
+            // Update remaining time estimate
             auto elapsed = timer.elapsed();
             QMetaObject::invokeMethod(this,
                 [this, elapsed, batchSize] { committedBatch(elapsed, batchSize); },
@@ -113,8 +119,12 @@ void FileContentIndexer::slotStartedIndexingFile(const QString& filePath)
     }
 }
 
-void FileContentIndexer::slotFinishedIndexingFile(const QString& filePath)
+void FileContentIndexer::slotFinishedIndexingFile(const QString& filePath, bool fileUpdated)
 {
+    if (fileUpdated) {
+        m_updatedFiles.append(filePath);
+    }
+
     m_currentFile = QString();
     if (!m_registeredMonitors.isEmpty()) {
         Q_EMIT finishedIndexingFile(filePath);
@@ -140,3 +150,5 @@ void FileContentIndexer::monitorClosed(const QString& service)
     m_registeredMonitors.removeAll(service);
     m_monitorWatcher.removeWatchedService(service);
 }
+
+#include "moc_filecontentindexer.cpp"
